/*
  gpsdialog.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "gpsdialog.h"
#include "mainwindow.h"
#include "qgconf.h"
#include <qlayout.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qstring.h>
#include <qspinbox.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcombobox.h>

#define DIALOG_WIDTH  300
#define DIALOG_HEIGHT 150


GpsDialog::GpsDialog( QWidget* parent, const char* name,
						  bool modal, WFlags fl)
	: QDialog( parent, name, modal, fl )
{
	resize( QSize(DIALOG_WIDTH, DIALOG_HEIGHT).expandedTo(minimumSizeHint()) );

	setCaption(QString::fromUtf8("GPS設定"));
	setFont(QFont("lcfont", 16));

	QVBoxLayout *base_vbox = new QVBoxLayout(this);
	selecter = new QComboBox(this);
	for(int i = 0; i < 5; i++){
		QString str;
		str.sprintf("GPS Device %d", i);
		selecter->insertItem(str);
	}
	base_vbox->addWidget(selecter);

	QGrid *grid = new QGrid(2, this);

	new QLabel(QString::fromUtf8("Name:"), grid);
	this->name = new QLineEdit(grid);

	new QLabel(QString::fromUtf8("GPS Device:"), grid);
	device = new QLineEdit(grid);

	new QLabel(QString::fromUtf8("PreCommand:"), grid);
	preCommand = new QLineEdit(grid);

	new QLabel(QString::fromUtf8("PostCommand:"), grid);
	postCommand = new QLineEdit(grid);

	new QLabel(QString::fromUtf8("更新間隔:"), grid);
	interval = new QSpinBox(1, 10, 1, grid);

	new QLabel(QString::fromUtf8("測地系:"), grid);
	QButtonGroup *datum = new QButtonGroup(2, Qt::Horizontal, grid);
	datum->setFrameStyle(QFrame::NoFrame);
	datum->setMargin(0);
	tokyo_datum = new QRadioButton(QString::fromUtf8("Tokyo"), datum);
	wgs84_datum = new QRadioButton(QString::fromUtf8("WGS84"), datum);
	wgs84_datum->setChecked(true);

	new QLabel(QString::fromUtf8("時差:"), grid);
	tz = new QComboBox(grid);
	for(int i = 14; i>= -11; i--){
		QString str;

		str.sprintf("UTC%+d", i);
		tz->insertItem(str);
	}

	base_vbox->addWidget(grid);

#ifndef QWS
	QHBox *hbox = new QHBox(this);
	QPushButton *cancel = new QPushButton("Cancel", hbox);
	QPushButton *ok = new QPushButton("OK", hbox);
	ok->setDefault(true);

	connect(cancel, SIGNAL(clicked()),
			this, SLOT(reject()));
	connect(ok, SIGNAL(clicked()),
			this, SLOT(accept()));

	base_vbox->addWidget(hbox);
#endif
	connect(selecter, SIGNAL(activated(int)),
			this, SLOT(changeDevice(int)));
	connect(this->name, SIGNAL(textChanged(const QString &)),
			this, SLOT(changeName(const QString &)));

	now_index = -1;
}

GpsDialog::~GpsDialog()
{
}

void GpsDialog::setInterval(int interval)
{
	this->interval->setValue(interval);
}

void GpsDialog::setDatum(Datum datum)
{
	switch(datum){
	case Tokyo:
		tokyo_datum->setChecked(true);
		break;
	case WGS84:
		wgs84_datum->setChecked(true);
		break;
	}
}

void GpsDialog::setTimeZone(int timezone)
{
	tz->setCurrentItem(14-timezone);
}

int GpsDialog::getInterval(void)
{
	return interval->text().toInt();
}

Datum GpsDialog::getDatum(void)
{
	return tokyo_datum->isChecked()?Tokyo:WGS84;
}

int GpsDialog::getTimeZone(void)
{
	return 14 - tz->currentItem();
}

void GpsDialog::loadConfig()
{
	Enter();

	QGConf *c = (QGConf *)lookupObject(this, "QGConf", "config");
	if(!c)
		return;

	int i;
	for(i = 0; i < MAXGPSSETTINGS; i++){
		QString group;
		group.sprintf("GPSDevice%d", i + 1);
		c->setGroup(group);

		configs[i].name = c->readEntry("Name", group);
		configs[i].device = c->readEntry("Device", DEFAULT_GPS_DEVICE);
		configs[i].preCommand = c->readEntry("PreCommand", DEFAULT_PRE_COMMAND);
		configs[i].postCommand = c->readEntry("PostCommand", DEFAULT_POST_COMMAND);
		configs[i].interval = c->readNumEntry("Interval", DEFAULT_GPS_INTERVAL);
		configs[i].datum = (Datum)c->readNumEntry("Datum", DEFAULT_GPS_DATUM);
		configs[i].timezone = c->readNumEntry("TimeZone", DEFAULT_GPS_TIMEZONE);

#ifdef DEBUG
		printf("name = %s\n", configs[i].name.ascii());
		printf("device = %s\n", configs[i].device.ascii());
		printf("preC = %s\n", configs[i].preCommand.ascii());
		printf("postC = %s\n", configs[i].postCommand.ascii());
		printf("interval = %d\n", configs[i].interval);
		printf("datum = %d\n", configs[i].datum);
		printf("timezone = %d\n", configs[i].timezone);
#endif
		selecter->changeItem(configs[i].name, i);
	}
	c->setGroup("");

	changeDevice(0);
}

void GpsDialog::saveConfig()
{
	Enter();

	QGConf *c = (QGConf *)lookupObject(this, "QGConf", "config");
	if(!c)
		return;

	int i;
	for(i = 0; i < MAXGPSSETTINGS; i++){
		QString group;
		group.sprintf("GPSDevice%d", i + 1);
		c->setGroup(group);

		c->writeEntry("Name", configs[i].name);
		c->writeEntry("Device", configs[i].device);
		c->writeEntry("PreCommand", configs[i].preCommand);
		c->writeEntry("PostCommand", configs[i].postCommand);
		c->writeEntry("Interval", configs[i].interval);
		c->writeEntry("Datum", configs[i].datum);
		c->writeEntry("TimeZone", configs[i].timezone);
	}
	c->setGroup("");
}

void GpsDialog::storeItems(int index)
{
	configs[index].name = name->text();
	configs[index].device = device->text();
	configs[index].preCommand = preCommand->text();
	configs[index].postCommand = postCommand->text();
	configs[index].interval = getInterval();
	configs[index].datum = getDatum();
	configs[index].timezone = getTimeZone();
}

void GpsDialog::loadItems(int index)
{
	name->setText(configs[index].name);
	device->setText(configs[index].device);
	preCommand->setText(configs[index].preCommand);
	postCommand->setText(configs[index].postCommand);
	setInterval(configs[index].interval);
	setDatum(configs[index].datum);
	setTimeZone(configs[index].timezone);
}

void GpsDialog::changeName(const QString &str)
{
	selecter->changeItem(str, selecter->currentItem());
}

void GpsDialog::changeDevice(int index)
{
	Enter();
	if(now_index >= 0)
		storeItems(now_index);

	now_index = index;
	loadItems(index);
}

void GpsDialog::accept()
{
	if(now_index >= 0)
		storeItems(now_index);
	QDialog::accept();
}
