/*
  layeredbase.h
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include "qgmap.h"
#include "layeredbase.h"
#include "layeredwidget.h"
#include <qwidget.h>
#include <qpixmap.h>
#include <qapplication.h>
#include <stdio.h>

LayeredBase::LayeredBase( QWidget *parent, const char *name, WFlags fl) :
	QWidget(parent, name, fl)
{
	clicked_layer = -1;
	paint_request = false;
	initialized = false;
	dbl_click_time.start();
	dbl_click = false;
}

void LayeredBase::addWidget( LayeredWidget *widget )
{
	int i;
	i = array.size();
	array.resize(i+1);
	array[i] = widget;
}

void LayeredBase::paintEvent( QPaintEvent *e UNUSED)
{
	if(!initialized)
		return;
	Enter();

	QPixmap scr(this->size());
	uint i;

	for(i=0;i<array.count();i++){
		if(!array[i]->isHidden() &&
		   e->rect().intersects(array[i]->geometry()) )
			array[i]->paintEvent(e, scr);
	}

	bitBlt(this, e->rect().topLeft(), &scr, e->rect());
}

void LayeredBase::showEvent(QShowEvent *e UNUSED)
{
	Enter();
	uint i;
	for(i=0; i < array.count(); i++){
		array[i]->showEvent();
	}
	initialized = true;
}

void LayeredBase::keyPressEvent ( QKeyEvent * e )
{
	int i;

	// default state is ignore
	e->ignore();

	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];
		if(w->isHidden())
			continue;
		if(w->keyPressEvent(e)){
			// set the accept flag, if the key event is accepted.
			e->accept();
			break;
		}
	}
	return;
}

void LayeredBase::keyReleaseEvent ( QKeyEvent * e )
{
	int i;

	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];
		if(w->isHidden())
			continue;
		if(w->keyReleaseEvent(e)){
			break;
		}
	}
	return;
}

void LayeredBase::wheelEvent( QWheelEvent *e )
{
	Enter();
	int i;

	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];
		if(w->isHidden())
			continue;
		if(w->wheelEvent(e))
			break;
	}
	return;
}

void LayeredBase::mousePressEvent( QMouseEvent *e )
{
	Enter();
	int i;

	press_pos = e->pos();
	press_time.start();

	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];
		if(w->isHidden())
			continue;
		if(w->geometry().contains(e->pos())){
			if(w->mousePressEvent(e)){
				clicked_layer = i;
				break;
			}
		}
	}
}

void LayeredBase::mouseMoveEvent( QMouseEvent *e )
{
	Enter();
	if(clicked_layer<0)
		return;

	array[clicked_layer]->mouseMoveEvent(e);
}

void LayeredBase::mouseReleaseEvent( QMouseEvent *e )
{
	if (clicked_layer >= 0) {
		array[clicked_layer]->mouseReleaseEvent(e);
		clicked_layer = -1;
	}

	if (press_time.elapsed() >= 200 ||
		(e->pos() - press_pos).manhattanLength() > 5)
		return;

//	printf("time:%d length:%d\n", dbl_click_time.elapsed(),
//		   (first_press_pos - press_pos).manhattanLength());
	dbl_click = (dbl_click_time.elapsed() < 500 &&
				 (first_press_pos - press_pos).manhattanLength() <= 40);
//	printf("%sclick\n", dbl_click ? "dbl ":"");
	dbl_click_time.start();
	if (!dbl_click)
		first_press_pos = press_pos;

	for (int i = array.count()-1; i >= 0; i--) {
		LayeredWidget *w = array[(uint)i];
		if (w->isHidden())
			continue;
		if (w->geometry().contains(e->pos())) {
			if (dbl_click) {
				if(w->mouseDoubleClickEvent(e)){
					break;
				}
			} else {
				if (w->mouseClickEvent(e)) {
					break;
				}
			}
		}
	}

}

void LayeredBase::mouseDoubleClickEvent( QMouseEvent *e )
{
	Enter();
	int i;

	mousePressEvent(e);
#if 0
	for(i = array.count()-1; i >= 0; i--){
		LayeredWidget *w = array[(uint)i];
		if(w->isHidden())
			continue;
		if(w->geometry().contains(e->pos())){
			if(w->mouseDoubleClickEvent(e)){
				clicked_layer = i;
				break;
			}
		}
	}
#endif
}

void LayeredBase::customEvent( QCustomEvent *e )
{
	switch ((int)e->type()) {
	case EV_CUSTOM_REPAINT:
	{
		CustomRepaintEvent *ev = (CustomRepaintEvent *)e;
		paint_request = false;
		if (ev->rect().isValid())
			QWidget::repaint(ev->rect(), FALSE);
		else
			QWidget::repaint(FALSE);
		break;
	}
	default:
		break;
	}
}

void LayeredBase::repaint()
{
	if (paint_request) {
//		printf("repaint blocked.\n");
		return;
	}

//	printf("repaint queued.\n");
	paint_request = true;

	pApp->postEvent(this, new CustomRepaintEvent(QRect()));
}

void LayeredBase::repaint(int x UNUSED, int y UNUSED, int w UNUSED, int h UNUSED)
{
	repaint();
}

void LayeredBase::sendRequest(QString const &req, QString const &arg)
{
	int i;
	for(i=0;i<(int)array.count();i++)
		array[i]->request(req, arg);
}
