/*
  map_gmap.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG
#include "qgmap.h"
#include <qobject.h>
#include <math.h>
#include "pos.h"
#include "map.h"
#include "map_gmap.h"

MapItemGMap::MapItemGMap(Map *parent) :
	mappath(parent), img_cache(15,15)
{
	parent->addItem(this);

	connect(parent, SIGNAL(configChanged(const QString &, const QString &)),
			this, SLOT(setConfig(const QString &, const QString &)));

	setGamma(1.0);

	max_down = 4;

	img_cache.setAutoDelete(TRUE);

	noimage.load(NOIMG_FILE);
}

MapItemGMap::~MapItemGMap()
{
	Enter();
	img_cache.clear();
}

bool MapItemGMap::drawItem(QImage &scr, const Pos &pos)
{
	Enter();

	Pos org_pos = pos;
	Pos work_pos;
	QString org_path, path;
	QString id, org_id;
	int i=0;

	work_pos = org_pos;

	org_id = pos.serialize();
	org_path = mappath.getPath(org_pos);

	QImage *img_p;
	img_p = img_cache.find(org_id);
	if(img_p){
		scr = img_p->copy();
		return true;
	}

	// load picture
	img_p = new QImage;
	if(org_path != "" && img_p->load(org_path)){
		gammaCorrection(img_p);
		img_cache.insert(org_id, img_p);
		scr = img_p->copy();
		return true;
	}
	delete img_p;

	while( i++ < max_down ){

		if(!work_pos.zoomup(1)){
			// noimage
			break;
		}

		id = work_pos.serialize();
		path = mappath.getPath(work_pos);
		if(path == "")
			continue;

		QImage *work_p;

		// find cache
		work_p = img_cache.find(id);
		if( !work_p ){
			work_p = new QImage();

			if (!work_p->load(path))
				continue;
			gammaCorrection(work_p);
			img_cache.insert(id, work_p);
		}

		QImage *img = zoom(work_p, i, org_pos);
		img_cache.insert(org_id, img);
		scr = img->copy();
		return true;

	}

	scr = noimage.copy();
	return false;
}

void MapItemGMap::setConfig(const QString &key, const QString &value)
{
	Enter();

	if(key=="Gamma"){
		setGamma(value.toDouble());
//		emit reload(true);
	}
}

void MapItemGMap::setGamma(double gamma)
{
	Enter();
	int i;

	for(i = 0; i < 256; i++){
		gamma_table[i] = (int)(255.0 * pow((double)i/255.0, 1.0/gamma) + 0.5);
	}

	img_cache.clear();
}

void MapItemGMap::gammaCorrection(QImage *img)
{
	Enter();
	int num = img->numColors();
	QRgb before, after;
	int i;

	for(i=0; i<num;i++){
		before = img->color(i);
		after = qRgb(gamma_table[qRed(before)],
					 gamma_table[qGreen(before)],
					 gamma_table[qBlue(before)]);
		img->setColor(i, after);
	}
}

QImage *MapItemGMap::zoom(QImage *img, int z, Pos const &pos)
{
	Enter();
	if(img->depth() < 8){
		img->convertDepth(8);
	}

	QImage *ret = new QImage(img->copy());
	int x,y;
	int pos_x, pos_y;
	int offset_x, offset_y;

	unsigned char *src;
	unsigned char *dst;

	int zoom = 1 << z;

	pos_x = (pos.getX() / IMAGE_SIZE) & (zoom-1);
	pos_y = (pos.getY() / IMAGE_SIZE) & (zoom-1);

	offset_x = IMAGE_SIZE / zoom * pos_x;
	offset_y = IMAGE_SIZE / zoom * pos_y;

	if(img->depth() != 8){
		printf("depth %d is not supported\n", img->depth());
		return ret;
	}

	for(y=0; y<img->height(); y++){
		dst = ret->scanLine(y);
		src = img->scanLine(offset_y + (y>>z));
		for(x=0;x<img->width();x++){
			*dst++ = src[offset_x + (x>>z)];
		}
	}
	return ret;
}

